import { asyncDelay, asyncTweenBy, asyncTweenTo, onHomeworkFinish, playDragonArmature, RandomInt } from "../script/util";
import { MyCocosSceneComponent } from "../script/MyCocosSceneComponent";

const { ccclass, property } = cc._decorator;

@ccclass
export default class SceneComponent extends MyCocosSceneComponent {

    addPreloadImage() {
        // this._imageResList.push({ url: this.data.pic_url });
    }

    addPreloadAudio() {
        // this._audioResList.push({ url: this.data.audio_url });
    }

    addPreloadAnima() {

    }

    onLoadEnd() {
        this.initData();
        this.initView();
        this.initListener();
    }

    initData() {
    }

    initAudio() {
    }

    bgmId;
    bgmInterval;
    startBgm(startTime = 0) {
        const bgm = cc.find('bgm').getComponent(cc.AudioSource);
        const bgmId = cc.audioEngine.play(bgm.clip, false, 1);
        this.bgmId = bgmId;
        cc.audioEngine.setCurrentTime(bgmId, startTime);
        this.bgmInterval = setInterval(() => {
            var time = cc.audioEngine.getCurrentTime(bgmId);
            this.onPlayBgmInTime(time);
        }, 10);
    }
    stopBgm() {
        clearInterval(this.bgmInterval);
        cc.audioEngine.stop(this.bgmId);
    }
    jumpBgm(time) {
        cc.audioEngine.setCurrentTime(this.bgmId, time);
    }

    onPlayBgmInTime(time) {
        for (let i = this._timeLine.length - 1; i >= 0; i--) {
            const timeLine = this._timeLine[i];
            if (timeLine.time < time) {
                if (!timeLine.done) {
                    const result = timeLine.func();
                    if (!result) {
                        timeLine.done = true;
                    }
                }
                break;
            }
        }
    }

    _timeLine = [];
    addTimeLine(time, func) {
        this._timeLine.push({ time: time, func: func, done: false });
    }

    async initView() {
        this.flyFlys();
        console.log('this.data.step = ' + this.data.step);
        if (this.data.step == 0) {
            this.step1();
        } else if (this.data.step == 1) {
            await this.jumpToStep1();
            await this.step2();
        } else if (this.data.step == 2) {
            await this.jumpToStep2();
            await this.step3();
        }
    }

    flyFlys() {
        const Fly_Blue_FlyNode = cc.find('Canvas/bg/Fly_Blue/Fly_Blue');
        const Fly_Green_FlyNode = cc.find('Canvas/bg/Fly_Green/Fly_Green');
        const Fly_Bell_FlyNode = cc.find('Canvas/bg/Fly_Bell/Fly_Bell');
        const Fly_Orange_FlyNode = cc.find('Canvas/bg/Fly_Orange/Fly_Orange');
        const Fly_Purple_FlyNode = cc.find('Canvas/bg/Fly_Purple/Fly_Purple');

        const flyRandom = (node) => {
            cc.tween(node)
                .to(0.5 + 0.5 * Math.random(), { x: RandomInt(-100, 100), y: RandomInt(-100, 100) })
                .call(() => {
                    flyRandom(node);
                })
                .start();
        };
        flyRandom(Fly_Blue_FlyNode);
        flyRandom(Fly_Green_FlyNode);
        flyRandom(Fly_Bell_FlyNode);
        flyRandom(Fly_Orange_FlyNode);
        flyRandom(Fly_Purple_FlyNode);
    }

    step1() {
        this.startBgm(0);
        // 一开始画面中没有精灵、圣诞老人和各种礼物。
        // 炉火燃烧，

        // 镜头从左向右移动，
        this.addTimeLine(12, () => {
            const camera = cc.find('Canvas/Main Camera');
            cc.tween(camera)
                .delay(0.5)
                .to(2.5, { x: 1000 }, { easing: 'quadInOut' })
                .start();
            // 圣诞老人从画面左侧走进来，
            const christmasMan = cc.find('Canvas/bg/ChristmasMan');
            asyncTweenTo(christmasMan, 3.4, { x: 1430 });
        });

        this.addTimeLine(15.4, () => {
            // 拍手召唤精灵，同时嘴巴张合。
            this.christmasManClap();
        })

        this.addTimeLine(19.4, () => {
            // PS01拼单词 配time\to\work这三个单词
            this.stopBgm();
            this.callQuestion('PS02');
        })
    }

    async jumpToStep1() {
        const camera = cc.find('Canvas/Main Camera');
        camera.x = 1000;
        const christmasMan = cc.find('Canvas/bg/ChristmasMan');
        christmasMan.x = 1430;
    }

    async step2() {
        this.startBgm(15);

        this.addTimeLine(15.4, () => {
            // 拍手召唤精灵，同时嘴巴张合。
            this.christmasManClap();
        });

        // 精灵从画面左侧飞进来
        const flyGreen = cc.find('Canvas/bg/Fly_Green');
        const flyBell = cc.find('Canvas/bg/Fly_Bell');
        const flyPurple = cc.find('Canvas/bg/Fly_Purple');
        const flyBlue = cc.find('Canvas/bg/Fly_Blue');
        const flyOrange = cc.find('Canvas/bg/Fly_Orange');
        this.addTimeLine(17, () => {
            [flyBlue, flyGreen, flyBell, flyOrange, flyPurple].forEach((fly) => {
                cc.tween(fly)
                    .delay(Math.random() * 0.3)
                    .by(6, { x: 3430 }, { easing: 'quadInOut' })
                    .start();
            });

            // 拿着礼物和铃铛的在天上飞 【没有拿礼物的精灵素材】
            [flyGreen, flyBell, flyPurple].forEach(fly => {
                cc.tween(fly)
                    .delay(3 + 1 * Math.random())
                    .by(1, { y: 200 })
                    .by(1, { y: -200 })
                    .start();
            });

            // 拿着星星和蝴蝶结的在地上跳动
            [flyBlue, flyOrange].forEach(fly => {
                cc.tween(fly)
                    .delay(4 + 1 * Math.random())
                    .by(0.5, { y: 200 })
                    .by(0.5, { y: -200 })
                    .delay(0.5 + 2 * Math.random())
                    .by(0.5, { y: 200 })
                    .by(0.5, { y: -200 })
                    .start();
            });
        });


        this.addTimeLine(26, () => {
            // 自23.7秒开始一个精灵拖着礼物箱子进来，其他精灵绕着飞。
            const box = cc.find('Canvas/bg/box');
            box.active = true;
            asyncTweenTo(box, 4, { x: 300 });
        });

        this.addTimeLine(30, () => {
            // 圣诞树上下弹动。【没有单独的圣诞树素材】
            // 圣诞老人转头上下打量圣诞树，
            // 又转向精灵，
            // 然后嘴巴张合，双手挥动两次。
            this.christmasManSee();
        });

        this.addTimeLine(32, () => {
            // 精灵们上下飞动寻找
            const flyGreenFlyNode = cc.find('Canvas/bg/Fly_Green/Fly_Green');
            const flyBellFlyNode = cc.find('Canvas/bg/Fly_Bell/Fly_Bell');
            const flyPurpleFlyNode = cc.find('Canvas/bg/Fly_Purple/Fly_Purple');
            const flyBlueFlyNode = cc.find('Canvas/bg/Fly_Blue/Fly_Blue');
            const flyOrangeFlyNode = cc.find('Canvas/bg/Fly_Orange/Fly_Orange');
            const boxFly_0 = cc.find('Canvas/bg/box/Fly_Green_0/Fly_Green');
            const boxFly_1 = cc.find('Canvas/bg/box/Fly_Green_1/Fly_Green');
            const boxFly_2 = cc.find('Canvas/bg/box/Fly_Green_2/Fly_Green');
            const boxFly_3 = cc.find('Canvas/bg/box/Fly_Green_3/Fly_Green');
            [
                flyGreenFlyNode,
                flyBellFlyNode,
                flyPurpleFlyNode,
                flyBlueFlyNode,
                flyOrangeFlyNode,
                boxFly_0,
                boxFly_1,
                boxFly_2,
                boxFly_3
            ].forEach((fly) => {
                fly.stopAllActions();
                const randomFly = (node) => {
                    cc.tween(node)
                        .to(0.5 + 0.5 * Math.random(), { x: RandomInt(-500, 500), y: RandomInt(-500, 500) })
                        .call(() => {
                            randomFly(node);
                        })
                        .start();
                }
                randomFly(fly);
            });
        });

        this.addTimeLine(32.7, () => {
            // （写字母S）
            this.callQuestion('PS01');
        });

        this.addTimeLine(38.5, () => {
            this.jumpBgm(33.5);
            return true;
        });
    }

    async christmasManSee() {
        const clap = cc.find('Canvas/bg/ChristmasMan/ChristmasMan_0');
        clap.active = true;
        clap.scaleX = -1;
        await playDragonArmature(clap, 'christmas_look', 1);
    }
    async jumpToStep2() {
        const flyGreen = cc.find('Canvas/bg/Fly_Green');
        const flyBell = cc.find('Canvas/bg/Fly_Bell');
        const flyPurple = cc.find('Canvas/bg/Fly_Purple');
        const flyBlue = cc.find('Canvas/bg/Fly_Blue');
        const flyOrange = cc.find('Canvas/bg/Fly_Orange');

        [flyBlue, flyGreen, flyBell, flyOrange, flyPurple].forEach((fly) => {
            fly.x += 3430;
        });

        const box = cc.find('Canvas/bg/box');
        box.active = true;
        box.x = 300;
        const camera = cc.find('Canvas/Main Camera');
        camera.x = 1000;

        const christmasMan = cc.find('Canvas/bg/ChristmasMan');
        christmasMan.x = 1430;
    }
    async step3() {
        this.startBgm(30);

        this.addTimeLine(30.2, () => {
            this.christmasManSee();
        })

        // 精灵们上下飞动寻找
        const flyGreenFlyNode = cc.find('Canvas/bg/Fly_Green/Fly_Green');
        const flyBellFlyNode = cc.find('Canvas/bg/Fly_Bell/Fly_Bell');
        const flyPurpleFlyNode = cc.find('Canvas/bg/Fly_Purple/Fly_Purple');
        const flyBlueFlyNode = cc.find('Canvas/bg/Fly_Blue/Fly_Blue');
        const flyOrangeFlyNode = cc.find('Canvas/bg/Fly_Orange/Fly_Orange');
        const boxFly_0 = cc.find('Canvas/bg/box/Fly_Green_0/Fly_Green');
        const boxFly_1 = cc.find('Canvas/bg/box/Fly_Green_1/Fly_Green');
        const boxFly_2 = cc.find('Canvas/bg/box/Fly_Green_2/Fly_Green');
        const boxFly_3 = cc.find('Canvas/bg/box/Fly_Green_3/Fly_Green');

        this.addTimeLine(32, () => {
            [
                flyGreenFlyNode,
                flyBellFlyNode,
                flyPurpleFlyNode,
                flyBlueFlyNode,
                flyOrangeFlyNode,
                boxFly_0,
                boxFly_1,
                boxFly_2,
                boxFly_3
            ].forEach((fly) => {
                fly.stopAllActions();
                cc.tween(fly)
                    .to(0.5 + 0.5 * Math.random(), { x: RandomInt(-500, 500), y: RandomInt(-500, 500) })
                    .to(0.5 + 0.5 * Math.random(), { x: RandomInt(-500, 500), y: RandomInt(-500, 500) })
                    .to(0.5 + 0.5 * Math.random(), { x: RandomInt(-500, 500), y: RandomInt(-500, 500) })
                    .to(0.5 + 0.5 * Math.random(), { x: RandomInt(-500, 500), y: RandomInt(-500, 500) })
                    .to(0.5 + 0.5 * Math.random(), { x: RandomInt(-500, 500), y: RandomInt(-500, 500) })
                    .to(0.5 + 0.5 * Math.random(), { x: 0, y: 0 })
                    .start();
            });
        });

        this.addTimeLine(38, () => {
            // 最终一个精灵举着星星飞到圣诞老人手里，同时星星明暗闪烁。
            const star = cc.find('Canvas/bg/box/star');
            const tree = cc.find('Canvas/bg/tree');
            const flyOrange = cc.find('Canvas/bg/Fly_Orange');
            const boxStarPos = flyOrange.parent.convertToNodeSpaceAR(
                star.convertToWorldSpaceAR(cc.v2(0, 0))
            );
            const treeStarPos = flyOrange.parent.convertToNodeSpaceAR(
                tree.convertToWorldSpaceAR(cc.v2(0, 1000))
            );
            const flyOrangeWithStar = cc.find('Canvas/bg/Fly_Orange/Fly_Orange/Fly_Orange_Fly');
            const flyOrangeWithoutStar = cc.find('Canvas/bg/Fly_Orange/Fly_Orange/Fly_Orange_Fly2');
            flyOrange.stopAllActions();
            const flyOrangePos = {
                x: flyOrange.x,
                y: flyOrange.y
            }
            cc.tween(flyOrange)
                .to(0.5, { x: boxStarPos.x, y: boxStarPos.y })
                .call(() => {
                    flyOrangeWithStar.active = true;
                    flyOrangeWithoutStar.active = false;
                    const boxStar = cc.find('Canvas/bg/box/star');
                    boxStar.active = false;
                })
                .to(1, { x: treeStarPos.x, y: treeStarPos.y })
                .call(() => {
                    flyOrangeWithStar.active = false;
                    flyOrangeWithoutStar.active = true;
                    const treeStar = cc.find('Canvas/bg/tree/star');
                    treeStar.active = true;
                    const treeStarLight = cc.find('Canvas/bg/tree/star/star2');
                    cc.tween(treeStarLight)
                        .set({ opacity: 0 })
                        .to(1, { opacity: 255 })
                        .to(1, { opacity: 0 })
                        .union()
                        .repeatForever()
                        .start();
                })
                .to(1, { x: flyOrangePos.x, y: flyOrangePos.y })
                .start();
        });

        this.addTimeLine(41.5, () => {
            // （语音评测跟读star,angel,bell,bow）
            this.stopBgm();
            this.callQuestion('PS03');
        });


        // // 所有精灵飞舞庆祝。
        // [
        //     flyGreenFlyNode,
        //     flyBellFlyNode,
        //     flyPurpleFlyNode,
        //     flyBlueFlyNode,
        //     flyOrangeFlyNode,
        //     boxFly_0,
        //     boxFly_1,
        //     boxFly_2,
        //     boxFly_3
        // ].forEach((fly) => {
        //     fly.stopAllActions();
        //     cc.tween(fly)
        //         .to(0.5 + 0.5 * Math.random(), { x: RandomInt(-500, 500), y: RandomInt(-500, 500) })
        //         .to(0.5 + 0.5 * Math.random(), { x: RandomInt(-500, 500), y: RandomInt(-500, 500) })
        //         .to(0.5 + 0.5 * Math.random(), { x: RandomInt(-500, 500), y: RandomInt(-500, 500) })
        //         .to(0.5 + 0.5 * Math.random(), { x: RandomInt(-500, 500), y: RandomInt(-500, 500) })
        //         .to(0.5 + 0.5 * Math.random(), { x: RandomInt(-500, 500), y: RandomInt(-500, 500) })
        //         .to(0.5 + 0.5 * Math.random(), { x: 0, y: 0 })
        //         .start();
        // });
        // await asyncDelay(5);

    }
    async christmasManTakeStar() {
        const walk = cc.find('Canvas/bg/ChristmasMan/ChristmasMan_2');
        walk.active = false;

        const star = cc.find('Canvas/bg/ChristmasMan/star')
        star.active = true;

        const clap = cc.find('Canvas/bg/ChristmasMan/ChristmasMan_0');
        clap.active = true;
        await playDragonArmature(clap, 'normal', 1);
        const starOnTheTree = cc.find('Canvas/bg/star');
        starOnTheTree.active = true;
        star.active = false;
    }

    async christmasManClap() {
        const walk = cc.find('Canvas/bg/ChristmasMan/ChristmasMan_2');
        walk.active = false;

        const clap = cc.find('Canvas/bg/ChristmasMan/ChristmasMan_0');
        clap.active = true;
        await playDragonArmature(clap, 'christmas_clap', 1);
    }

    async callQuestion(questionName) {
        const courseIdMap = {
            PS01: 21256,
            PS02: 21257,
            PS03: 21258,
        }
        const mask = cc.find('Canvas/Main Camera/mask');
        const hand = cc.find(`Canvas/Main Camera/${questionName}`);
        hand.active = true;
        mask.active = true;
        hand.on('click', () => {
            hand.active = false;
            mask.active = false;
            const middleLayerNode = cc.find('middleLayer');
            if (middleLayerNode) {
                const middleLayer = middleLayerNode.getComponent('middleLayer');
                // middleLayer.loadOnlineBundle(questionName);
                middleLayer.loadOnlineBundleByCourseId(courseIdMap[questionName]);
            }
        });
        cc.tween(mask)
            .set({ opacity: 0 })
            .to(0.5, { opacity: 80 })
            .start();
        cc.tween(hand)
            .delay(2)
            .to(0.3, { angle: 30 })
            .to(0.6, { angle: -30 })
            .to(0.3, { angle: 0 })
            .union()
            .repeatForever()
            .start();
    }

    initListener() {

    }
}
